###################################################
### chunk number 1: 
###################################################
options(keep.source=FALSE)
options(width=175)
options(continue="  ")


###################################################
### chunk number 2: 
###################################################
setClass("RegressionObject",
         representation(data="data.frame",
                        variableNameDictionary="list",
                        formulas="list",
                        lags="list",
                        caption="character",
                        label="character",
                        rhsVariables="character",
                        rowNames="character",
                        columnNames="character",
                        numberOfFormulas="numeric",
                        numberOfRhsVariables="numeric",
                        numberOfRows="numeric",
                        formulaStrings="list",
                        results="list",
                        test="ANY",
                        alignString="character",
                        table="matrix",
                        xtable="character",
                        sidewaysTable="character"),
         prototype(formulaStrings=list(c()),
                   results=list(list(c()),list(c())),
                   xtable='...Space for x-table...',
                   rhsVariables=c('...'),
                   columnNames=c(' '),
                   sidewaysTable='table',
                   alignString='ll|ccccccccccccccccccccccccccccc')
         )


###################################################
### chunk number 3: 
###################################################
setGeneric('CreateLaggedVariables',
           function(object, FNUM) { 
             standardGeneric('CreateLaggedVariables')
           })

setMethod(f = "CreateLaggedVariables",
          signature = "RegressionObject",
          definition = function(object, FNUM) {

            LHS_VARIABLE <- object@formulas[[FNUM]][[1]]
            ls.rhsVariables <- c()
            
            NUMBER_OF_RHS_VARIABLES <- length(object@formulas[[FNUM]][[2]])
            for (VNUM in 1:NUMBER_OF_RHS_VARIABLES) {
              for (LAG in object@lags[[FNUM]]) {
                
                RHS_VARIABLE <- object@formulas[[FNUM]][[2]][VNUM]
                if (LAG > 0) RHS_VARIABLE <- paste(RHS_VARIABLE,'.lag',LAG,sep='')
                
                if (RHS_VARIABLE != LHS_VARIABLE) {
                  
                  ls.rhsVariables <- c(ls.rhsVariables,RHS_VARIABLE)
                  
                }
              } 
            }
            
            return(ls.rhsVariables)
            
            
          })


setGeneric('CreateFormulaStrings',
           function(object, FNUM) { 
             standardGeneric('CreateFormulaStrings')
           })

setMethod(f = "CreateFormulaStrings",
          signature = "RegressionObject",
          definition = function(object, FNUM) {
            
            LHS_VARIABLE <- object@formulas[[FNUM]][[1]]
            FORMULA <- LHS_VARIABLE
            
            ls.rhsVariables <- CreateLaggedVariables(object,FNUM)
            
            for (RHS_VARIABLE in ls.rhsVariables) {
              
              SEPARATOR <- ' + ' 
              if (FORMULA == LHS_VARIABLE) SEPARATOR <- ' ~ '
              
              FORMULA <- paste(FORMULA,RHS_VARIABLE,sep=SEPARATOR)
            }
            
            return(FORMULA)
            
          })


###################################################
### chunk number 4: 
###################################################
setGeneric('CreateOutputTable',
           function(object) { 
             standardGeneric('CreateOutputTable')
           })

setMethod(f = "CreateOutputTable",
          signature = "RegressionObject",
          definition = function(object) {
            
            NUMBER_OF_ROWS <- 2 + 2*object@numberOfRows + 3
            NUMBER_OF_COLUMNS <- object@numberOfFormulas + 1
            
            m.outputTable <- ones(NUMBER_OF_ROWS,NUMBER_OF_COLUMNS)*NA
            
            m.outputTable[1:2,1] <- c('(Intercept)',' ')
            for (VNUM in 1:object@numberOfRows) {
              m.outputTable[(VNUM*2+1):(VNUM*2+2),1] <- c(object@rowNames[VNUM],' ')
            }
            m.outputTable[NUMBER_OF_ROWS-2,1] <- 'N'
            m.outputTable[NUMBER_OF_ROWS-1,1] <- '$R^2$'
            m.outputTable[NUMBER_OF_ROWS,1] <- 'Adj. $R^2$'
            
            ls.columnNames <- c(' ')
            for (COLUMN in 2:NUMBER_OF_COLUMNS) {
              ls.columnNames <- c(ls.columnNames,paste('(',COLUMN-1,')',sep=''))
            }
            dimnames(m.outputTable)[[2]] <- ls.columnNames
            
            return(m.outputTable)
            
          })


###################################################
### chunk number 5: 
###################################################
setGeneric('FillInOutputTable',
           function(object) { 
             standardGeneric('FillInOutputTable')
           })

setMethod(f = "FillInOutputTable",
          signature = "RegressionObject",
          definition = function(object) {
     
            FormatEstimates <- function(ls.results) {
              
              COEFFICIENT <- ls.results[1]
              SE          <- ls.results[2]
              P_VALUE     <- ls.results[4]
              
              STARS <- ''
              if (P_VALUE<.05) STARS <- '*'
              if (P_VALUE<.01) STARS <- '**'
              
              ls.formattedCoefficientAndTstat <- c(paste(signif(COEFFICIENT,2),STARS,sep=''),
                                                   paste('(',signif(SE,2),')',sep=''))
              
              return(ls.formattedCoefficientAndTstat)
              
            }
            
            m.outputTable <- object@table
            
            for (COLUMN in 1:object@numberOfFormulas) {
              NUMBER_OF_RHS_VARIABLES <- dim(object@results[[2]][[COLUMN]]$coef)[1]
              for (ROW in 1:NUMBER_OF_RHS_VARIABLES) {
                VARIABLE_NAME <- rownames(object@results[[2]][[COLUMN]]$coef)[ROW]
                ROW_NAME <- object@variableNameDictionary[[2]][object@variableNameDictionary[[1]]==VARIABLE_NAME]
                ROW_NAME_CLEAN <- FormatRowNames(object,VARIABLE_NAME)
                OUTPUT_ROW <- which(m.outputTable[,1] == ROW_NAME_CLEAN)
                ls.outputRows <- c(OUTPUT_ROW,OUTPUT_ROW+1)
                m.outputTable[ls.outputRows,COLUMN+1] <- FormatEstimates(object@results[[2]][[COLUMN]]$coef[ROW,])
              }
              
              NUMBER_OF_ROWS <- dim(m.outputTable)[1]
              m.outputTable[NUMBER_OF_ROWS-2,COLUMN+1] <- length(object@results[[1]][[COLUMN]]$residuals)
              m.outputTable[NUMBER_OF_ROWS-1,COLUMN+1] <- signif(object@results[[2]][[COLUMN]]$r.squared,3)
              m.outputTable[NUMBER_OF_ROWS,COLUMN+1] <- signif(object@results[[2]][[COLUMN]]$adj.r.squared,3)
            }
            
            return(m.outputTable)
            
          })            


###################################################
### chunk number 6: 
###################################################
setGeneric('FormatRowNames',
           function(object,VAR) { 
             standardGeneric('FormatRowNames')
           })

setMethod(f = "FormatRowNames",
          signature = "RegressionObject",
          definition = function(object, VAR) {
            
            if (substr(VAR,nchar(VAR)-3,nchar(VAR)-1) == 'lag') {
              VAR_BASE <- substr(VAR,1,nchar(VAR)-5)
              VAR_LAG <- substr(VAR,nchar(VAR),nchar(VAR))
            } else {
              VAR_BASE <- VAR
              VAR_LAG <- NA
            }
            VAR_CLEAN <- object@variableNameDictionary[[2]][which(VAR_BASE==object@variableNameDictionary[[1]])]
            if (VAR == "(Intercept)") {
              VAR_CLEAN <- VAR 
            } else if (is.na(VAR_LAG)) { 
              VAR_CLEAN <- paste('$',gsub('TIME','t',VAR_CLEAN),'$',sep='') 
            } else if (!is.na(VAR_LAG)) {
              VAR_CLEAN <- paste('$',gsub('TIME',paste('t-',VAR_LAG,sep=''),VAR_CLEAN),'$',sep='')
            }
            
            return(VAR_CLEAN)
            
          })



###################################################
### chunk number 7: 
###################################################
setMethod(f = "initialize",
          signature = "RegressionObject",
          function(.Object, ...) {
            
            .Object <- callNextMethod()            
            
            .Object@numberOfFormulas <- length(.Object@formulas)
            if (length(.Object@lags) == 1 & .Object@numberOfFormulas>1) {
              for (REG in 1:.Object@numberOfFormulas) {
                .Object@lags[[REG]] <- .Object@lags[[1]]
              }
            }
            
            for (FNUM in 1:.Object@numberOfFormulas){
              
              .Object@formulaStrings[[FNUM]] <- CreateFormulaStrings(.Object,FNUM)
              .Object@results[[1]][[FNUM]] <- lm(.Object@formulaStrings[[FNUM]], .Object@data)
              .Object@results[[2]][[FNUM]] <- summary(.Object@results[[1]][[FNUM]] )
              .Object@columnNames <- c(.Object@columnNames,.Object@formulas[[FNUM]][[1]])
              
              .Object@rhsVariables <- unique(c(.Object@rhsVariables,CreateLaggedVariables(.Object,FNUM)))
              
            }
            
            .Object@rhsVariables <- .Object@rhsVariables[!(.Object@rhsVariables %in% c('','+','...'))]
            .Object@numberOfRhsVariables <- length(.Object@rhsVariables)
            .Object@rowNames <- .Object@rhsVariables
            for (VAR in .Object@rowNames) {
              .Object@rowNames[.Object@rowNames==VAR] <- FormatRowNames(.Object,VAR)
            }            
            .Object@rowNames <- unique(.Object@rowNames)
            .Object@numberOfRows <- length(.Object@rowNames)
            
            .Object@table <- CreateOutputTable(.Object)
            .Object@table <- FillInOutputTable(.Object)
            
            ALIGN <- substr(.Object@alignString,1,3+.Object@numberOfFormulas)
            ls.horizontalLines <- c(-1,0,dim(.Object@table)[1]-3,dim(.Object@table)[1])
            
            .Object@xtable <- print(xtable(.Object@table, 
                                           caption=.Object@caption, 
                                           label=.Object@label,
                                           align=ALIGN),
                                    include.rownames=FALSE,
                                    hline.after=ls.horizontalLines,
                                    floating=TRUE,
                                    type="latex",
                                    table.placement="H",
                                    floating.environment=.Object@sidewaysTable,
                                    sanitize.text.function = function(x){x})
            
            return(.Object)

          })



